/* File: CellTowers.cpp
 *
 * A program that determines the maximum population that can be covered
 * by purchasing cell towers with a single transmitting frequency.
 */
#include <iostream>
#include <algorithm>
#include "vector.h"
#include "console.h"
using namespace std;

/* Given a list of cities, returns the maximum number of people we can provide
 * cell service to given that no two adjacent cities can have coverage.
 */
int bestCoverageFor(const Vector<int>& cities) {
    /* Base case: If there are no cities, we can cover a whopping zero people. */
    if (cities.isEmpty()) {
        return 0;
    } else {
        /* Option 1: DON'T include the first city. In that case, we want to maximize
         * the coverage to everyone else.
         */
        int without = bestCoverageFor(cities.subList(1, cities.size() - 1));
        
        /* Option 2: DO include the first city. Then we can't cover the second, and we
         * want to maximize coverage to all the remaining cities.
         */
        int with = cities[0]; // Coverage from the first city.
        if (cities.size() >= 2) {
            with += bestCoverageFor(cities.subList(2, cities.size() - 2));
        }
        
        /* One of those two options is better. Return whichever one that is! */
        return max(with, without);
    }
}

int main() {
    Vector<int> populations = { 137, 106, 107, 166, 103, 261, 109 };

    /* Want to see how the recursion works? Put a breakpoint on this line! */
    int coverage = bestCoverageFor(populations);
    cout << "Best coverage: " << coverage << endl;

	return 0;
}


